'' =================================================================================================
''
''   File....... jd_animation__2017-08-25.spin
''   Purpose.... Jeff David Animation Demo
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2011-2017 Jon McPhalen
''   E-mail..... 
''   Started.... 
''   Updated.... 25 AUG 2017
''
'' =================================================================================================


con

  ' Critical: The revision letter must match the PCB that will receive this program. There were
  '           several design updates to the Rev F PCB.

  PCB_REV = "E"


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = _xinfreq * (_clkmode >> 6)                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us
                                                                 
  EFX_SLOW =    2_400                                           ' EFX-TEK baud rates                                             
  EFX_FAST =   38_400                                              

  EFX_BAUD = EFX_SLOW                                           ' for older AP-16s                                                     
  TRM_BAUD =  115_200                                           ' terminal baud rate

  MS_ADJ   = MS_001 * 75 / 100                                  ' adjusted ms for frame timing

                                                                 
con { io pins }                                                  
                                                                 
  RX1     = 31  { I }                                           ' serial / programming
  TX1     = 30  { O }                                            
                                                                 
  SDA1    = 29  { I/O }                                         ' i2c / eeprom
  SCL1    = 28  { I/O }                                          

  XP0     = 27  { I/O }                                         ' extra port 0 - not on Rev E or lower
  XP1     = 24  { I/O }                                         ' extra port 1 - not on Rev E or lower 
  XP2     = 19  { I/O }                                         ' extra port 2 - not on Rev E or lower    
  XP3     = 16  { I/O }                                         ' extra port 3 - not on Rev E or lower 
                                                         
  OUT_EN  = 27  { O }                                           ' outputs enable pin (low)     
                                                                 
  SIO     = 26  { I/O }                                         ' TTL serial io
                                                              
  SIO_RX  = 26  { I/O }                                         ' TTL serial io
  SIO_TX  = 25  { I/O }                                         ' TTL serial io (not enabled)  
                                                             
  RX2     = 23  { I }                                           ' RS-485 port
  RX2_EN  = 22  { O }                                                 
  TX2_EN  = 21  { O }                                                 
  TX2     = 20  { O }

  R_LED   = 18  { O }                                           ' bi-color LED (pins may need swap)
  G_LED   = 17  { O }                                                 
                                                             
  OUT7    = 15  { O }                                           ' outputs
  OUT6    = 14  { O }                                                 
  OUT5    = 13  { O }                                                 
  OUT4    = 12  { O }                                                 
  OUT3    = 11  { O }                                                 
  OUT2    = 10  { O }                                                 
  OUT1    =  9  { O }                                                 
  OUT0    =  8  { O }                                                 
                                                                 
  OPT_BR  =  7  { I/O }                                         ' options switches
  OPT_A1  =  6  { I/O }                                                  
  OPT_A0  =  5  { I/O }                                                  
  OPT_SM  =  4  { I/O }                                                  
                                                                 
  DMX_A8  =  3  { do not use with Rev C and earlier PCBs }      ' high-bit of DMX address
                                                                 
  DO_165  =  2  { I }                                           ' control lines for 74x165
  CLK_165 =  1  { O }                                                  
  LD_165  =  0  { O }                                                  

  
  ' for EFX-TEK uSD adapter module (#99002)
  ' -- must open all option switches when installed

  SD_CS   =  OPT_BR  '   7                                  
  SD_DI   =  OPT_A1  '   6
  SD_CLK  =  OPT_A0  '   5
  SD_DO   =  OPT_SM  '   4


con { app io }

  CHANNELS = 5                                                  ' PWM on OUT0..OUT4

  ' driver indexes

  TURN    =  2 {OUT7}                                           ' servos 
  TILT    =  1 {OUT6}                                            
  NOD     =  0 {OUT5}                                                  
                                                                 
  SPOTLT  =  1 {OUT1}                                           ' pwm                                        
  EYES    =  0 {OUT0}                                           ' digital
                                                                 
                                                                 
con

  #(true), ON, OFF                                              ' digital control
                                                                 
  #0, NO, YES                                                    
                                                                 
  #0, BLK, RED, GRN, YEL                                        ' for r/g led
                                                                 
  #0, NOCON, RX, TX, RXTX                                       ' for rs-485
                                                                 
  #0, LSBFIRST, MSBFIRST                                         

  
  DEBUG = NO
  
                                                            
obj                              
                                 
' main                                                          ' * master Spin cog                   
  time       : "jm_time_80"                                     '   timing and delays (80MHz system) 
  prng       : "jm_prng"                                        '   pseudo-random number generator     
  io         : "jm_io"                                          '   essential io
  servo      : "jm_servo8"                                      ' * servo driver 
  outs       : "jm_pwm8"                                        ' * pwm driver for outputs
  term       : "jm_fullduplexserial"                            ' * serial IO for terminal
  serial     : "jm_fullduplexserial"                            ' * serial IO for AP-16+ & EFX-TEK devices
  sd         : "fsrw"                                           ' * uSD file IO
' background                                                    ' * background RGY LED control, inputs scanning 
  
' * uses cog when loaded
                                 
                                 
var

  word  idle[3]                                                 ' idle positions

  ' do not change order or type!!!

  word  nodpos                                                  ' servo positions
  word  tiltpos
  word  turnpos
  word  frtiming                                                ' convert to ms with x20


dat

  ServoFile     byte    "servos.txt", 0
  AudioFile     byte    "sfx00", 0                              ' extension not required for WAV


con { behavior spec }

{

  I have 3 servos on (Nod, Tilt,Turn)
  LCD Eye power on OUT 0
  LED spotlight power OUT 1 PWM
  Serial IO to AP-16+
   
  1. PWM the LED spotlight
  2. Power on the LCD eyes
  3. Start playing SFX.WAV
  4. Read data and position servos Nod,Tilt,Turn X times 20ms see data.csv  (add smoothing code data2.csv)
 ?5.  Loop through all data checking if audio stops.  Start data over if audio still playing
  6.  Power off LCD eyes
  7.  PWM LED spotlight to off
  8.  Wait 2 minutes
  9.  Loop back to top


  Link to audio:
    -- https://www.dropbox.com/s/bp6f4ywaiij1yag/sfx00_mono_32k.wav?dl=0
    -- file converted from stero, 44.1kHz to mono, 32kHz

}
                                                                 
                                                      
pub main                                              
                                                                 
  setup                                                         ' setup io and objects
  
  verify_servo_file(@ServoFile)

  ap16_stop(%00)

  if (DEBUG == YES)
    term.rxflush
    term.rx
    term.tx(term#CLS)
  
  repeat
    if (DEBUG == YES)
      term.tx(term#CLS)
      term.str(string("Starting show.", 13, 13))
  
    fade_spotlight(0, 255, 2000)                                ' #1 
    outs.high(EYES)                                             ' #2 
    ap16_wav(%00, @AudioFile, 1)                                ' #3 
    play_show(@ServoFile)                                       ' #4

    ' optional
    ' -- must use 2400 baud with G command on older AP-16s
    {
    repeat while (ap16_playing(%00))                            ' #5
      time.pause(20)
    }
        
    outs.low(EYES)                                              ' #6
    fade_spotlight(255, 0, 2000)                                ' #7
    time.pause(120_000)                                         ' #8


pub verify_servo_file(p_str) | ch

'' Verifies presence of tile and reads idle positions

  ch := open_file(@ServoFile, "r")                              ' try to open file
  
  if (ch == false)
    set_rgy_flash(RED, 500, BLK, 500)                           ' error flash

    if (DEBUG == YES)
      term.str(string("-- Error: Servo file not found."))

    repeat                                                      ' wait here for correction
      waitcnt(0)        

  rd_line(@linebuf, 80)                                         ' read first line (must have values!)
  replace(@linebuf, 13, 0)                                      ' prep for parsing

  repeat ch from 0 to 2 
    idle[ch] := extract_dec(field(ch, p_str, ",")) * 10        ' extract & covert to us 

  set_idle_position
    

pub set_idle_position | ch

'' Move to idle position
'' -- moves slowly in case called from middle of show

  repeat ch from 0 to 2
    servo.set(ch, idle[ch], 900)                                ' slow move (1/2s) to idle 

      
pub fade_spotlight(first, last, delay) | t, level

'' Fade spotlight from first level to last level in delay milliseconds
'' -- first & last are DMX-compatible levels (0..255)

   delay := (delay * MS_001) / (||(last-first) + 1)             ' convert delay to system ticks 

   t := cnt                                                     ' sync to system timer
   repeat level from first to last                              ' loop through levels
     outs.set(SPOTLT, outs.gamma8(level))                       ' set LED brightness
     waitcnt(t += delay)  


pub play_show(p_str) | check, t, frtix, line, ch

'' Play servo show from uSD card
'' -- p_str is pointer to name of file to play

  check := open_file(@ServoFile, "r")                           ' try to open file
  
  if (check == false)
    set_rgy_solid(RED)                                          ' problem with file

    if (DEBUG == YES)
      term.str(string("-- Error: Servo file not found."))
      
    return                                                      ' abort

  set_rgy_flash(GRN, 63, BLK, 62)                               ' play flash 

  time.start
  line := 0
  
  t := cnt                                                      ' sync to system timer
  repeat
    check := rd_line(@linebuf, 80)                              ' read line from file 
    if (check =< 0)                                             ' if EOF, break out of loop           
      quit                         
    else
      replace(@linebuf, 13, 0)                                  ' strip CR from input line   
      parse_values(@linebuf)                                    ' parse data from it         
      servo.set(NOD,  nodpos , 0)                                                      
      servo.set(TILT, tiltpos, 0)                                                      
      servo.set(TURN, turnpos, 0)
      
      time.mark

      if (DEBUG == YES)
        term.rjdec(++line,   4, " ")                            ' show line from file
        term.rjdec(nodpos,   8, " ")
        term.rjdec(tiltpos,  6, " ")
        term.rjdec(turnpos,  6, " ")
        term.rjdec(frtiming, 8, " ")
        term.tx(13)
      
      waitcnt(t += (frtiming * MS_ADJ))                         ' real-time delay (adjusted)               

  t := time.millis                                              ' capture show time
    
  close_file

  set_idle_position
  
  set_rgy_solid(BLK)                                            ' led off

  if (DEBUG == YES)
    term.str(string(13, "Show time = "))   
    term.dec(t / 1000)               
    term.tx(".")                               
    term.rjdec(t // 1000, 3, "0")    
    term.str(string(" seconds"))               


pub replace(p_str, char, fix)

'' Replace instances of char with fix inside string at p_str

  repeat strsize(p_str)
    if (byte[p_str] == char)
      byte[p_str] := fix
    ++p_str

    
pub parse_values(p_str) | idx

'' Extract four byte-wide values from data file

  nodpos   := extract_dec(field(0, p_str, ",")) * 10            ' convert to microseconds 
  tiltpos  := extract_dec(field(1, p_str, ",")) * 10
  turnpos  := extract_dec(field(2, p_str, ",")) * 10 
  frtiming := extract_dec(field(3, p_str, ",")) * 20            ' convert to milliseconds 


pub extract_dec(p_str) : value | c

'' Extract decimal value from string at p_str
'' -- stops on encountering non-digit character

  repeat
    c := byte[p_str++]
    if ((c => "0") and (c =< "9"))
      value *= 10
      value += c - "0"
    else
      quit


pub field(fn, p_str, sep) | c

'' Return pointer to desired field in string
'' -- fn is field number (0..n)
'' -- p_str is pointer to string to search
'' -- sep is field separator character 

  if (fn > ccount(p_str, sep))
    return -1                                                   ' error

  if (fn > 0)
    repeat
      if (byte[p_str++] == sep)
        if (--fn == 0)
          quit   

  return p_str
   

pub ccount(p_str, c) : n

'' Counts the occurances of c in string at p_str

  repeat strsize(p_str)
    if (byte[p_str++] == c)
      ++n

                                                           
pub setup
                                                                 
'' Configure HC-8+ IO and support objects                        
                                                                 
  time.start                                                    ' setup for timing/delays
                                                                 
  prng.seed(-cnt, -cnt ~> 2, $CAFE_BABE, cnt <- 2, cnt)         ' seed randomizer                             
                                                                 
  io.start(0, 0)                                                ' clear pins (this cog)

  if (PCB_REV =< "E")                                  
    io.low(OUT_EN)                                              ' enable output driver
                                                               
  outs.start(CHANNELS, OUT0)                                    ' start outputs driver *

  servo.start(3, OUT5)                                          ' start servo driver *
                                                                 
  set_rs485(NOCON)                                              ' RS-485 not connected

  if (DEBUG == YES)
    term.start(RX1, TX1, %0000, TRM_BAUD)                       ' start serial for terminal *
  
  ap16_setup(SIO, EFX_BAUD)                                     ' start serial for AP-16+ *   

  mount_sd                                                      ' start sd access *
                                                    
  bcog := cognew(background, @bcstack) + 1                      ' start background cog *


con

  { --------------------------------- }
  {  S U P P O R T   R O U T I N E S  } 
  { --------------------------------- }

                                                           
pub ttl_pin | check                                                    
                                                                 
'' Reads TTL states for 1-of-8 input                             
'' -- returns -1 (no input)                                      
'' -- returns 0 to 7 for input on IN0 to IN7                     
''    * only one input allowed

  check := ttlpins                                  
                                                                 
  if (bit_count(check) == 1)                                    ' only one button pressed?
    return bit_pos(check, 0)                                    '  yes, return button #
  else                                                           
    return -1                                                   '  none or bad input
                                                                 

pub ttl_inputs 

'' Return TTL inputs status
'' -- pins are scanned by background cog

  return ttlpins


pub dmx_address 

'' Return DMX address switch setting
'' -- switches are scanned by background cog
'' -- Note: For Rev C and lower boards, do not use DMX 256 switch -- leave it OFF

  return dmxaddr


pub read_xport                                                  ' Rev F and higher only
                                                                 
'' Returns state of xport bits
'' -- does not set dira bits to 0
'' -- pins are not contiguous!  

  if (PCB_REV < "F")
    return %0000
  else
    return (ina[XP3] << 3) | (ina[XP2] << 2) | (ina[XP1] << 1) | ina[XP0]


pub write_xport(value, outsmask)                                ' Rev F and higher only    
                                                                 
'' Write value to xport pins                                     
'' -- outsmask used to set xport output pins                     
'' -- will not change pins with 0 bit in outsmask                
                                                                 
  if (PCB_REV < "F")                                             
    return

  ' unrolled
  ' -- pins are not contiguous!

  if (outsmask & %1000)
    io.set(XP3, value >> 3)  

  if (outsmask & %0100)
    io.set(XP2, value >> 2)

  if (outsmask & %0010)
    io.set(XP1, value >> 1)    

  if (outsmask & %0001)
    io.set(XP0, value >> 0)

    
pub set_rgy_solid(color)                                              
                                                                  
'' Sets R/G LED                                                   
'' -- direct control only                                         
'' -- red/green/yellow + blinking requires support cog   
                                                                  
  ifnot (bcog)                                                  ' if support cog not running
    dira[R_LED..G_LED] := %11                                   ' make outputs
    case color                                                  ' set color to state                  
      BLK : outa[R_LED..G_LED] := %00                                                                 
      RED : outa[R_LED..G_LED] := %10                                                                 
      GRN : outa[R_LED..G_LED] := %01                                                                 
      YEL : outa[R_LED..G_LED] := %00                           ' yellow requires active driver       

  else                                                           
    case color                                                   
      BLK : set_rgy_flash(BLK, 100, BLK, 100)                   ' set to solid color 
      RED : set_rgy_flash(RED, 100, RED, 100)                          
      GRN : set_rgy_flash(GRN, 100, GRN, 100)                           
      YEL : set_rgy_flash(YEL, 100, YEL, 100)


pub set_rgy_flash(p0c, p0ms, p1c, p1ms)                                 
                                                                  
'' Explicit (control registers) setup of OGRY LED
'' -- requires support cog to be running               
'' -- p0c is phase 0 color (OFF, GRN, RED, YEL)                   
'' -- p0ms is phase 0 timing in milliseconds                      
'' -- p1c is phase 1 color                                        
'' -- p1ms is phase 1 timing in milliseconds

  ifnot (bcog)
    return                     
                                                                  
  if ((p0c => OFF) and (p0c =< YEL))                            ' phase 0 color
    rgycolor[0] := p0c                                           
  else                                                           
    rgycolor[0] := OFF                                           
                                                                 
  if (p0ms > 0)                                                 ' phase 0 timing                                           
    rgytime[0] := p0ms                                           
  else                                                           
    rgytime[0] := 100                                            
                                                                 
  if ((p1c => OFF) and (p1c =< YEL))                            ' phase 1 color                             
    rgycolor[1] := p1c                                           
  else                                                           
    rgycolor[1] := OFF                                           
                                                                 
  if (p1ms > 0)                                                 ' phase 1 timing                                              
    rgytime[1] := p1ms                                           
  else                                                            
    rgytime[1] := 100                                             

                                                           
pub set_rs485(state)                                              
                                                                  
'' Sets RS-485 RX and TX enable inputs                            
                                                                  
  case state                                                      
    NOCON : outa[RX2_EN..TX2_EN] := %10                           
    RX    : outa[RX2_EN..TX2_EN] := %00                           
    TX    : outa[RX2_EN..TX2_EN] := %11                           
    RXTX  : outa[RX2_EN..TX2_EN] := %01                           
                                                                  
  dira[RX2_EN..TX2_EN] := %11                                     
                                                                  
                                                                  
pub bit_count(value) : bc                                        
                                                                  
'' Returns # of bits set (1) in value                             
                                                                  
  repeat while (value)                                          ' loop while there are 1s
    bc += (value & %1)                                          '  add bit value to count
    value >>= 1                                                 '  next bit
                                                                 
                                                                 
pub bit_pos(value, mode)                                         
                                                                 
'' Returns position of 1st bit set (0..31)                       
'' -- mode 0 to scan from lsb, mode 1 to scan from msb           
'' -- -1 = no bits set                                           
                                                                 
  if (value == 0)                                               ' if no bits                                   
    return -1                                                   '  return -1
  else                                                           
    if (mode == LSBFIRST)                                       ' check from LSB
      value ><= 32                                              '  flip for >|
      return (32 - >|value)                                      
    else                                                          
      return (>|value - 1)                                        
                                                                  
                                                                  
pub bit_val(value, pos)                                           
                                                            
'' Returns bit (0..1) from pos (0..31) in value                   
                                                                  
  if ((pos => 0) and (pos =< 31))                                 
    return (value >> pos) & 1                                     
  else                                                            
    return 0


con

  { ------------------------------- }
  {  A P - 1 6 +   R O U T I N E S  } 
  { ------------------------------- }

  PLAYING = 1 << 7                                              ' bit7 of status is playing bit


dat

  AP16                  byte    "!AP16", 0
  Version               byte    "???", 0


pub ap16_setup(pin, baud)

'' Start audio serial driver on pin at baud (half-duplex true mode)

  serial.start(pin, pin, %1100, baud)
  time.pause(10)
  serial.rxflush


pub ap16_version(addr) : idx | c

'' Returns version string from AP-16+ from device at addr

  serial.str(@AP16)
  serial.tx(addr)
  serial.tx("V")
  serial.rxflush

  bytemove(@Version, string("???"), 3)
  
  repeat idx from 0 to 2
    c := serial.rxtime(10)
    if (c < 0)
      quit
    else
      Version[idx] := c

  return @Version


pub ap16_status(addr)

'' Returns AP-16+ status byte

  serial.str(@AP16)
  serial.tx(addr)
  serial.tx("G")
  serial.rxflush

  return serial.rxtime(10) 


pub ap16_playing(addr) : status

'' Returns playing status from AP-16+ at addr
'' -- warning: returns false if no response from device

  status := ap16_status(addr)
  if (status < 0)
    return false
  else
    return ((status & PLAYING) == PLAYING)


pub ap16_sfx(addr, sfx, rpts)

'' Plays SFX file on AP-16+ at addr

  serial.str(@AP16)   
  serial.tx(addr)
  serial.str(string("PS"))
  serial.tx(sfx)
  serial.tx(rpts)


pub ap16_aux(addr, aux, rpts)

'' Plays AUX file on AP-16+ at addr

  serial.str(@AP16)   
  serial.tx(addr)
  serial.str(string("PA"))
  serial.tx(aux)
  serial.tx(rpts)


pub ap16_wav(addr, spntr, rpts)

'' Plays WAV file (name at spntr) on AP-16+ at addr

  serial.str(@AP16)   
  serial.tx(addr)
  serial.str(string("PW"))
  serial.str(spntr)
  serial.tx(13)
  serial.tx(rpts)


pub ap16_hold(addr, state)

'' Hold (state == 1) or release (state == 0) playing file
'' -- does not affect ambient loop

  serial.str(@AP16)   
  serial.tx(addr)
  serial.tx("H")
  if (state)
    serial.tx(1)
  else
    serial.tx(0)


pub ap16_relay(addr, state)

'' Set relay state (0..1) on AP-16+ at addr

  serial.str(@AP16)   
  serial.tx(addr)
  serial.tx("R")
  if (state)
    serial.tx(1)
  else
    serial.tx(0)
  

pub ap16_stop(addr)

'' Stops AP-16+ at addr

  serial.str(@AP16)   
  serial.tx(addr)
  serial.tx("X")


pub ap16_unload

'' Unloads serial driver for AP-16+
'' -- do not use if other devices on same serial link

  serial.stop

  
con                                                              
                                                                 
  { ----------------- }                                          
  {  F I L E   I / O  }                                          
  { ----------------- }                                          
                                                                 
                                                                 
var                                                              
                                                                 
  long  mounted                                                  
                                                                 
  byte  filename[8 + 1 + 3 + 1]                                 ' 8.3 plus 0
  byte  linebuf[80 + 1]                                         ' line buffer
                                                                 
                                                                 
pub mount_sd | check                                             
                                                                 
'' Attempt to mount SD card                                      
'' -- returns true if SD card successfully mounted

  check := \sd.mount_explicit(SD_DO, SD_CLK, SD_DI, SD_CS)

  mounted := (check == 0)   

  return mounted


pub dismount_sd

'' Unmounts SD card

  \sd.unmount
  mounted := false


pub has_file(p_str) | check

'' Returns true if file is on SD card

  check := \sd.popen(p_str, "r")
  if (check == 0)
    return true
  else
    return false 


pub open_file(p_str, mode) | check

'' Attempt to open file
'' -- p_str is pointer to filename (z-string)
'' -- mode is [lower-case] single character, (e.g., "r" for read)
'' -- sd card must be mounted before open

  if (mounted)                                                   
    check := \sd.popen(p_str, mode)                             ' attempt to open
    if (check == 0)                                              
      if (mode == "r")                                           
        \sd.seek(0)                                             ' force sector read 
      return true                                                
    else                                                         
      return false
  else
    return false


pub close_file

'' Closes open file

  \sd.pclose


pub rd_line(p_str, n) | c, len

'' Reads line of text from open text file
'' -- terminated by LF or EOF
'' -- p_str is address of string buffer
'' -- n is maximum number of characters to read from line

  len := 0                                                      ' index into string
                                                                 
  repeat n                                                       
    c := \sd.pgetc                                              ' get a character
    if (c < 0)                                                  ' end of file
      if (len == 0)                                             ' if nothing
        return -1                                               ' return empty
      else                                                       
        quit                                                     
                                                                 
    if (c <> 10)                                                ' if not LF char
      byte[p_str++] := c                                        '  add to buffer
      len++                                                      
    else                                                        ' if LF
      quit                                                      '  we're done
                                                                 
  byte[p_str] := 0                                              ' terminate end of line
                                                                 
  return len
  

con

  { ----------------------------- }
  {  B A C K G R O U N D   C O G  }
  {  - global milliseconds        }
  {  - R/G LED                    }
  {  - TTL/DMX inputs scanning    }
  { ----------------------------- }
  

var  

  long  bcog                                                    ' background cog #
  long  bcstack[32]                                             ' stack for background cog
                                                                 
  long  millis                                                  ' global milliseconds register
                                                                 
                                                                 
pri background | t                                              ' launch with cognew()
                                                                 
  io.low(R_LED)                                                 ' setup R/G LED pins
  io.low(G_LED)                                                  
                                                                 
  io.high(LD_165)                                               ' setup x165 io pins
  io.low(CLK_165)                                                
  io.input(DO_165)                                               
                                                                 
  millis := 0                                                    
                                                                 
  t := cnt                                                      ' sync loop timer
  repeat                                                         
    waitcnt(t += MS_001)                                        ' run loop every millisecond
    ++millis                                                     
    refresh_rg_led                                              ' update the R/G LED
    scan_ttl_ins                                                ' re-scan TTL & DMX inputs
                                                                 
                                                                 
var                                                              
                                                                 
  long  rgycolor[2]                                             ' led phase colors
  long  rgytime[2]                                              ' led phase timing (ms)
  long  rgphase                                                 ' current phase (red or green chip)
  long  phasetimer                                              ' time in phase
  long  rgtimer                                                 ' timer for rg process
                                                                 
                                                                 
pri refresh_rg_led                                              ' call only from support()
                                                                 
  if (++phasetimer => rgytime[rgphase])                         ' done with this phase?
    phasetimer := 0                                             ' yes, reset timer
    rgphase := 1 - rgphase                                      '  and invert phase
                                                                 
  if (++rgtimer == 16)                                           
    rgtimer := 0                                                 
                                                                 
  case rgycolor[rgphase]                                        ' set led to color for phase
    OFF:                                                         
      outa[R_LED..G_LED] := %00                                  
                                                                 
    GRN:
      outa[R_LED..G_LED] := %01  
                                                                 
    RED:
      outa[R_LED..G_LED] := %10 
                                                                   
    YEL:
      if (rgtimer < 2)                                                                       
        outa[R_LED..G_LED] := %10                               ' 1 red                        
      else                                                                                   
        outa[R_LED..G_LED] := %01                               ' 15                  
                                                          

var

  long  ttlpins
  long  dmxaddr                                                  

                                                                
pri scan_ttl_ins | tempin                                         
                                                                 
'' Scan TTL and DMX address inputs                               
                                                                 
  outa[LD_165] := 0                                             ' blip Shift/Load line    
  outa[LD_165] := 1                                              

  ' unrolled for best speed

  tempin := (tempin << 1) | ina[DO_165]                         ' bit15
  outa[CLK_165] := 1                                            ' blip clock  
  outa[CLK_165] := 0                                                        
  tempin := (tempin << 1) | ina[DO_165]
  outa[CLK_165] := 1                   
  outa[CLK_165] := 0                   
  tempin := (tempin << 1) | ina[DO_165]
  outa[CLK_165] := 1                   
  outa[CLK_165] := 0                   
  tempin := (tempin << 1) | ina[DO_165]
  outa[CLK_165] := 1                   
  outa[CLK_165] := 0
  tempin := (tempin << 1) | ina[DO_165] 
  outa[CLK_165] := 1                    
  outa[CLK_165] := 0                    
  tempin := (tempin << 1) | ina[DO_165] 
  outa[CLK_165] := 1                    
  outa[CLK_165] := 0                    
  tempin := (tempin << 1) | ina[DO_165] 
  outa[CLK_165] := 1                    
  outa[CLK_165] := 0
  tempin := (tempin << 1) | ina[DO_165] 
  outa[CLK_165] := 1                    
  outa[CLK_165] := 0
  
  tempin := (tempin << 1) | ina[DO_165]                         ' bit7
  outa[CLK_165] := 1                    
  outa[CLK_165] := 0                    
  tempin := (tempin << 1) | ina[DO_165] 
  outa[CLK_165] := 1                    
  outa[CLK_165] := 0                    
  tempin := (tempin << 1) | ina[DO_165] 
  outa[CLK_165] := 1                    
  outa[CLK_165] := 0                    
  tempin := (tempin << 1) | ina[DO_165] 
  outa[CLK_165] := 1                    
  outa[CLK_165] := 0
  tempin := (tempin << 1) | ina[DO_165] 
  outa[CLK_165] := 1                    
  outa[CLK_165] := 0                    
  tempin := (tempin << 1) | ina[DO_165] 
  outa[CLK_165] := 1                    
  outa[CLK_165] := 0                    
  tempin := (tempin << 1) | ina[DO_165] 
  outa[CLK_165] := 1                    
  outa[CLK_165] := 0                                       
  tempin := (tempin << 1) | ina[DO_165]                         ' bit0                    
                                                         
  ttlpins := tempin.byte[0]                                     ' update global vars
  dmxaddr := tempin.byte[1] | (ina[DMX_A8] << 8)     

                                                 
dat { license }                                                  

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}      